

/* ***********************************************************************
                Naval Air Warfare Center / Aircraft Division              
                        Patuxent Naval Air Station                         
 ========================================================================

   File: antenna.c                                         
                                                                        
   Classification: UNCLASSIFIED                                   
                                                                        
   Description:   This file contains the functions, methods, and global
      variables needed to extract and manipulate irregular tables in the TDB.
			
   Written by:	Alex Harper
*********************************************************************** */


#include <stdlib.h>
#include <string.h>
#include <math.h>

#include "antenna.h"


/* ***********************************************************
   This function returns a conversion factor that converts values
   defined in "start" units to values in "end" units

   char *start,   // initial unit type
   char *end      // final value type
*********************************************************** */
double get_scale(char *start, char *end) {

   double scale;

   // convert to HZ
   if (!strcmp(start, TOKEN_PHZP) || !strcmp(start, TOKEN_HZ))
      scale = 1.0;
   else if (!strcmp(start, TOKEN_PMHZP) || !strcmp(start, TOKEN_MHZ))
      scale = 1000000.0;
   else if (!strcmp(start, TOKEN_PGHZP) || !strcmp(start, TOKEN_GHZ))
      scale = 1000000000.0;

   else if (!strcmp(start, TOKEN_PRADIANSP) || !strcmp(start, TOKEN_RADIANS))
      scale = 1.0;
   else if (!strcmp(start, TOKEN_PDEGP) || !strcmp(start, TOKEN_DEG))
      scale = PI/180.0;

   // convert to meters
   else if (!strcmp(start, TOKEN_PMP) || !strcmp(start, TOKEN_M))
      scale = 1.0;
   else if (!strcmp(start, TOKEN_PKMP) || !strcmp(start, TOKEN_KM))
      scale = 1000.0;
   else if (!strcmp(start, TOKEN_PFTP) || !strcmp(start, TOKEN_FT))
      scale = 0.3048;
   else if (!strcmp(start, TOKEN_PFEETP) || !strcmp(start, TOKEN_FEET))
      scale = 0.3048;
   else if (!strcmp(start, TOKEN_PANGLESP) || !strcmp(start, TOKEN_ANGLES))
      scale = 304.8;
   else if (!strcmp(start, TOKEN_PMILESP) || !strcmp(start, TOKEN_MILES))
      scale = 1584.96;
   else if (!strcmp(start, TOKEN_PNMP) || !strcmp(start, TOKEN_NM))
      scale = 1852.0;

   // unknown
   else
      return 1.0;

   // convert from HZ
   if (!strcmp(end, TOKEN_PHZP) || !strcmp(end, TOKEN_HZ))
      scale *= 1.0;
   else if (!strcmp(end, TOKEN_PMHZP) || !strcmp(end, TOKEN_MHZ))
      scale *= 0.001;
   else if (!strcmp(end, TOKEN_PGHZP) || !strcmp(end, TOKEN_GHZ))
      scale *= 0.000001;

   else if (!strcmp(end, TOKEN_PRADIANSP) || !strcmp(end, TOKEN_RADIANS))
      scale *= 1.0;
   else if (!strcmp(end, TOKEN_PDEGP) || !strcmp(end, TOKEN_DEG))
      scale *= 180.0/PI;

   // convert from meters
   else if (!strcmp(end, TOKEN_PMP) || !strcmp(end, TOKEN_M))
      scale *= 1.0;
   else if (!strcmp(end, TOKEN_PKMP) || !strcmp(end, TOKEN_KM))
      scale *= 0.001;
   else if (!strcmp(end, TOKEN_PFTP) || !strcmp(end, TOKEN_FT))
      scale *= 3.2808399;
   else if (!strcmp(end, TOKEN_PFEETP) || !strcmp(end, TOKEN_FEET))
      scale *= 3.2808399;
   else if (!strcmp(end, TOKEN_PANGLESP) || !strcmp(end, TOKEN_ANGLES))
      scale *= 0.0032808399;
   else if (!strcmp(end, TOKEN_PMILESP) || !strcmp(end, TOKEN_MILES))
      scale *= 0.0006309307;
   else if (!strcmp(end, TOKEN_PNMP) || !strcmp(end, TOKEN_NM))
      scale *= 0.0005399568;

   // unknown
   else
      return 1.0;

   return scale;
}


/* ***********************************************************
  This function takes 2 vectors and calculates the elevation and
  azmuth difference between them

   double *bistatic,  // unit vector from target to receiver
   double *ux,        // unit x vector
   double *uz,        // unit z vector
   double *az,        // azimuth returned here
   double *el         // elevation returned here
*********************************************************** */
void VecAng(double *bistatic, double *ux, double *uz, double *az, double *el) {

   double rdux, rduy, rngazp;
   vector3d temp;

   rdux = dotproduct3(bistatic, ux);

   xproductd(temp, uz, ux);
   rduy = dotproduct3(bistatic, temp);

   if (fabs( rduy ) < 1.0e-15 )
      rduy = 0.0;
      
   temp[0] = rdux;
   temp[1] = rduy;
   rngazp = magnitude2(temp);

   if(rngazp > CORRECT) {
      *el = atan2(dotproduct3(bistatic,uz), rngazp);
      *az = atan2(rduy, rdux);
   }

   else
      *az = *el = 0.0;
}


/* ***********************************************************
  This function calculates the azmuth/elevation of a target from a receiver

   antenna_orientation_type *target, // target position/orientation data
   double *rec_pos,                   // receiver position
   double *xmit_pos,                  // transmitter position
   double *az,                        // returned azimuth
   double *el                         // returned elevation
*********************************************************** */
void calc_azel(antenna_orientation_type *target, double *rec_pos, double *xmit_pos, double *az, double *el) {

   vector3d work;
   vector3d bistatic;

   subeqarray3(bistatic, rec_pos, target->pos);
   normalize3d(bistatic);

   if (xmit_pos) {
      subeqarray3(work, xmit_pos, target->pos);
      normalize3d(work);
      addarray3(bistatic, work);
      normalize3d(bistatic);
   }
   
   VecAng(bistatic, target->forward, target->up, az, el);
}


/* ***********************************************************
  This method evaluates whether or not the input parameter lies
  w/in this interval

   double x  // value to search for
*********************************************************** */
int interval_type::query_between(double x) {

   return x >= min && (next ? x < max : x <= max);
}


/* ***********************************************************
   This method extracts a irregular table from the tdb file

   FILE *infile,    // input file
   char *token,     // last processed token
   int *errorflag   // error flag
*********************************************************** */
void table_type::extract_tdbdat(FILE *infile, char *token, int *errorflag) {

   double scale;
   interval_type *ptr;
   double start, end;

   intervalman.dest();
      
   // unit type 
   *errorflag = get_sweg_token(infile, token);

   if (!strcmp(token, TOKEN_PDEGP) || !strcmp(token, TOKEN_DEG))
      scale = get_scale(token, TOKEN_PRADIANSP);
   else if (!strcmp(token, TOKEN_PRADIANSP) || !strcmp(token, TOKEN_RADIANS))
      scale = 1.0;
   else if (!strcmp(token, TOKEN_PM2P) || !strcmp(token, TOKEN_M2))
      scale = 1.0;
   else if (!strcmp(token, TOKEN_PHZP) || !strcmp(token, TOKEN_HZ) ||
            !strcmp(token, TOKEN_PMHZP) || !strcmp(token, TOKEN_MHZ) ||
            !strcmp(token, TOKEN_PGHZP) || !strcmp(token, TOKEN_GHZ))
      scale = get_scale(token, TOKEN_PHZP);

   else
      scale = get_scale(token, TOKEN_PMP);

   // first frequency
   *errorflag = get_sweg_token(infile, token);
   end = atof(token)*scale;

   // second frequency
   *errorflag = get_sweg_token(infile, token);
   
   while (*errorflag) {

      if (!strcmp(token, TOKEN_DIMENSION)) {
	 
         for (ptr=(interval_type *)intervalman.head; ptr; ptr=(interval_type *)ptr->next) {
            // level
            *errorflag = get_sweg_token(infile, token);
            // type
            *errorflag = get_sweg_token(infile, token);

            if (!strcmp(token, TOKEN_AZ))
               ptr->data = new azimuth_type;
            else if (!strcmp(token, TOKEN_EL))
               ptr->data = new elevation_type;
            else if (!strcmp(token, TOKEN_ALT_XMIT))
               ptr->data = new alt_xmit_type;
            else if (!strcmp(token, TOKEN_ALT_RCVR))
               ptr->data = new alt_rcvr_type;
            else if (!strcmp(token, TOKEN_POL))
               ptr->data = new pol_type;
            else
               ptr->data = new dist_2d_type;

            ptr->data->extract_tdbdat(infile, token, errorflag);
         }
	 
         break;
      }

      if (!strcmp(token, TOKEN_GAIN)) {
         // unit type "DB"
         *errorflag = get_sweg_token(infile, token);
         *errorflag = get_sweg_token(infile, token);

         for (ptr=(interval_type *)intervalman.head; ptr; ptr=(interval_type *)ptr->next) {
            ptr->data = new gain_type;
            ptr->data->extract_tdbdat(infile, token, errorflag);
         }
	 
         break;
      }

      if (!strcmp(token, TOKEN_RCS) || !strcmp(token, TOKEN_SIGNATURE)) {
	 
         // unit type "M2"
         *errorflag = get_sweg_token(infile, token);
         *errorflag = get_sweg_token(infile, token);

         for (ptr=(interval_type *)intervalman.head; ptr; ptr=(interval_type *)ptr->next) {
            ptr->data = new rcs_type;
            ptr->data->extract_tdbdat(infile, token, errorflag);
         }
	 
         break;
      }

      start = end;
      end = atof(token)*scale;

/* asdf NOTE: only works on slices w/ only one interval - need to mirror image entire table
      if (fabs(start) < CORRECT && (query_whatami() == TABLE_ELEVATION || query_whatami() == TABLE_AZIMUTH))
         start = -end;
*/

      intervalman.append(ptr = new interval_type, NULL);
      ptr->min = start;
      ptr->max = end;      

      *errorflag = get_sweg_token(infile, token);
   }

}


/* ***********************************************************
   This method dumps various data elements to stdout

   char *prefix  // tab string to print before this current block of data
*********************************************************** */
void table_type::dump_data(char *prefix) {

   char buffer[MAXSTRLEN];
   interval_type *ptr;

   for (ptr=(interval_type *)intervalman.head; ptr; ptr=(interval_type *)ptr->next) {
      sprintf(buffer, "%s%f,%f\t ", prefix, ptr->min, ptr->max);
      ptr->data->dump_data(buffer);
   }
   
}


/* ***********************************************************
   This method traverses the frequency level of an irregular table
   for TRANSMISSION-LOSS

   double freq,         // frequency to search on
   double alt_xmit,     // transmitter altitude (MSL)
   double alt_rec,      // receiver altitude (MSL)
   double dis_2d        // ground distance between the two
*********************************************************** */
double frequency_type::query_data(double freq, double alt_xmit, double alt_rec, double dis_2d) {

   interval_type *ptr;

   if (!intervalman.head)
      return 0.0;

   if (freq < ((interval_type *)intervalman.head)->min)
      return ((interval_type *)intervalman.head)->data->query_data(freq, alt_xmit, alt_rec, dis_2d);
   
   for (ptr=(interval_type *)intervalman.head; ptr->next; ptr=(interval_type *)ptr->next)
      if (ptr->query_between(freq))
         return ptr->data->query_data(freq, alt_xmit, alt_rec, dis_2d);
	 
   return ptr->data->query_data(freq, alt_xmit, alt_rec, dis_2d);
}


/* ***********************************************************
   This method traverses the frequency level of an irregular table

   double freq,    // frequency to search on
   double az,      // azimuth to search on
   double el       // elevation to search on
*********************************************************** */
double frequency_type::query_data(double freq, double az, double el) {

   interval_type *ptr;

   if (!intervalman.head)
      return 1.0;

   if (freq < ((interval_type *)intervalman.head)->min)
      return ((interval_type *)intervalman.head)->data->query_data(freq, az, el);
   
   for (ptr=(interval_type *)intervalman.head; ptr->next; ptr=(interval_type *)ptr->next)
      if (ptr->query_between(freq))
         return ptr->data->query_data(freq, az, el);
	 
   return ptr->data->query_data(freq, az, el);
}


/* ***********************************************************
*********************************************************** */
void frequency_type::extract_data(dbl_llist_manager *man, double *az, double *el) {

// non applicable???
}


/* ***********************************************************
   This method traverses the frequency level of an irregular table
   to find the min/max frequency range

   double *data,        // data range to search
   double *az_range,    // azimuth range to search
   double *el_range     // elevation range to search
*********************************************************** */
void frequency_type::query_minmax(double *data, double *az_range, double *el_range) {

   interval_type *ptr;

   if (!intervalman.head)
      return;

   for (ptr=(interval_type *)intervalman.head; ptr; ptr=(interval_type *)ptr->next)
       ptr->data->query_minmax(data, az_range, el_range);
}


/* ***********************************************************
   This method traverses the azimuth level of an irregular table

   double freq,   // frequency to search
   double az,     // azimuth to search
   double el      // elevation to search
*********************************************************** */
double azimuth_type::query_data(double freq, double az, double el) {

   interval_type *ptr;

   if (!intervalman.head)
      return 1.0;

   if (az < ((interval_type *)intervalman.head)->min)
      return ((interval_type *)intervalman.head)->data->query_data(freq, az, el);
   
   for (ptr=(interval_type *)intervalman.head; ptr->next; ptr=(interval_type *)ptr->next)
      if (ptr->query_between(az))
         return ptr->data->query_data(freq, az, el);
	    
   return ptr->data->query_data(freq, az, el);
}


/* ***********************************************************
   Meets parent requirement
*********************************************************** */
double azimuth_type::query_data(double freq, double alt_xmit, double alt_rec, double dis_2d) {

   return 0;
}


/* ***********************************************************
   This method traverses the azimuth level of an irregular table

   dbl_llist_manager *man,  // az/el/data extracted list
   double *az_range,    // azimuth range to search
   double *el_range     // elevation range to search
*********************************************************** */
void azimuth_type::extract_data(dbl_llist_manager *man, double *az, double *el) {

   interval_type *ptr;
   azel_manager *aeman;
   
   if (!intervalman.head)
      return;

   for (ptr=(interval_type *)intervalman.head; ptr; ptr=(interval_type *)ptr->next) {
      man->append(aeman = new azel_manager, NULL);
      ptr->data->extract_data(&aeman->man, &ptr->min, el);

      man->append(aeman = new azel_manager, NULL);
      ptr->data->extract_data(&aeman->man, &ptr->max, el);
   }

}


/* ***********************************************************
   This method traverses the azimuth level of an irregular table

   double *data,        // data range to search
   double *az_range,    // azimuth range to search
   double *el_range     // elevation range to search
*********************************************************** */
void azimuth_type::query_minmax(double *data, double *az_range, double *el_range) {

   interval_type *ptr;

   if (!intervalman.head)
      return;

   if (az_range) {
      if (az_range[0] > ((interval_type *)intervalman.head)->min)
         az_range[0] = ((interval_type *)intervalman.head)->min;
	    
      if (az_range[1] < ((interval_type *)intervalman.tail)->max)
         az_range[1] = ((interval_type *)intervalman.tail)->max;
   }
   
   for (ptr=(interval_type *)intervalman.head; ptr; ptr=(interval_type *)ptr->next)
       ptr->data->query_minmax(data, az_range, el_range);
}


/* ***********************************************************
   This method traverses the elevation level of an irregular table

   double freq,   // frequency to search
   double az,     // azimuth to search
   double el      // elevation to search
*********************************************************** */
double elevation_type::query_data(double freq, double az, double el) {

   interval_type *ptr;

   if (!intervalman.head)
      return 1.0;

   if (el < ((interval_type *)intervalman.head)->min)
      return ((interval_type *)intervalman.head)->data->query_data(freq, az, el);

   for (ptr=(interval_type *)intervalman.head; ptr->next; ptr=(interval_type *)ptr->next)
      if (ptr->query_between(el))
         return ptr->data->query_data(freq, az, el);
	 
   return ptr->data->query_data(freq, az, el);
}


/* ***********************************************************
   Meets parent requirement
*********************************************************** */
double elevation_type::query_data(double freq, double alt_xmit, double alt_rec, double dis_2d) {

   return 0;
}


/* ***********************************************************
   This method traverses the elevation level of an irregular table

   dbl_llist_manager *man, // az/el/data extracted list
   double az,              // azimuth to search
   double el               // elevation to search
*********************************************************** */
void elevation_type::extract_data(dbl_llist_manager *man, double *az, double *el) {

   interval_type *ptr;
   azel_manager *aeman;
   
   if (!intervalman.head)
      return;

   for (ptr=(interval_type *)intervalman.head; ptr; ptr=(interval_type *)ptr->next) {
      man->append(aeman = new azel_manager, NULL);
      ptr->data->extract_data(&aeman->man, az, &ptr->min);

      man->append(aeman = new azel_manager, NULL);
      ptr->data->extract_data(&aeman->man, az, &ptr->max);
   }

}


/* ***********************************************************
   This method traverses the elevation level of an irregular table
   to find the min/max elevation levels

   double *data,        // data range to search
   double *az_range,    // azimuth range to search
   double *el_range     // elevation range to search
*********************************************************** */
void elevation_type::query_minmax(double *data, double *az_range, double *el_range) {

   interval_type *ptr;

   if (!intervalman.head)
      return;

   if (el_range) {
      if (el_range[0] > ((interval_type *)intervalman.head)->min)
         el_range[0] = ((interval_type *)intervalman.head)->min;
	    
      if (el_range[1] < ((interval_type *)intervalman.tail)->max)
         el_range[1] = ((interval_type *)intervalman.tail)->max;
   }
   
   for (ptr=(interval_type *)intervalman.head; ptr; ptr=(interval_type *)ptr->next)
       ptr->data->query_minmax(data, az_range, el_range);
}


/* ***********************************************************
   Meets parent requirement
*********************************************************** */
double alt_xmit_type::query_data(double freq, double az, double el) {

   return 0;
}


/* ***********************************************************
   This method traverses the transmitter altitude level of an irregular table
   for TRANSMISSION-LOSS

   double freq,         // frequency to search on
   double alt_xmit,     // transmitter altitude (MSL)
   double alt_rec,      // receiver altitude (MSL)
   double dis_2d        // ground distance between the two
*********************************************************** */
double alt_xmit_type::query_data(double freq, double alt_xmit, double alt_rec, double dis_2d) {

   interval_type *ptr;

   if (!intervalman.head)
      return 0.0;

   if (alt_xmit < ((interval_type *)intervalman.head)->min)
      return ((interval_type *)intervalman.head)->data->query_data(freq, alt_xmit, alt_rec, dis_2d);

   for (ptr=(interval_type *)intervalman.head; ptr->next; ptr=(interval_type *)ptr->next)
      if (ptr->query_between(alt_xmit))
         return ptr->data->query_data(freq, alt_xmit, alt_rec, dis_2d);
	 
   return ptr->data->query_data(freq, alt_xmit, alt_rec, dis_2d);
}


/* ***********************************************************
   This method just fills in the virtual function requirement for
   the alt_xmit_type class.

   double *data,        // data range to search
   double *az_range,    // azimuth range to search
   double *el_range     // elevation range to search
*********************************************************** */
void alt_xmit_type::query_minmax(double *data, double *az_range, double *el_range) {

}


/* ***********************************************************
   Meets parent requirement
*********************************************************** */
void alt_xmit_type::extract_data(dbl_llist_manager *man, double *az, double *el) {

}


/* ***********************************************************
   Meets parent requirement
*********************************************************** */
double alt_rcvr_type::query_data(double freq, double az, double el) {

   return 0;
}


/* ***********************************************************
   This method traverses the reciever altitude level of an irregular table
   to find the min/max altitude levels

   double freq,         // frequency to search on
   double alt_xmit,     // transmitter altitude (MSL)
   double alt_rec,      // receiver altitude (MSL)
   double dis_2d        // ground distance between the two
*********************************************************** */
double alt_rcvr_type::query_data(double freq,  double alt_xmit, double alt_rec, double dis_2d) {

   interval_type *ptr;

   if (!intervalman.head)
      return 0.0;

   if (alt_rec < ((interval_type *)intervalman.head)->min)
      return ((interval_type *)intervalman.head)->data->query_data(freq, alt_xmit, alt_rec, dis_2d);

   for (ptr=(interval_type *)intervalman.head; ptr->next; ptr=(interval_type *)ptr->next)
      if (ptr->query_between(alt_rec))
         return ptr->data->query_data(freq, alt_xmit, alt_rec, dis_2d);
	 
   return ptr->data->query_data(freq, alt_xmit, alt_rec, dis_2d);
}


/* ***********************************************************
   This method just fills in the virtual function requirement for
   the alt_rcver_type class.

   double *data,        // data range to search
   double *az_range,    // azimuth range to search
   double *el_range     // elevation range to search
*********************************************************** */
void alt_rcvr_type::query_minmax(double *data, double *az_range, double *el_range) {

}


/* ***********************************************************
   Meets parent requirement
*********************************************************** */
void alt_rcvr_type::extract_data(dbl_llist_manager *man, double *az, double *el) {

}


/* ***********************************************************
   Meets parent requirement
*********************************************************** */
double dist_2d_type::query_data(double freq, double az, double el) {

   return 0;
}


/* ***********************************************************
   This method traverses the ground distance level of an irregular table
   to find the min/max ground distance

   double freq,         // frequency to search on
   double alt_xmit,     // transmitter altitude (MSL)
   double alt_rec,      // receiver altitude (MSL)
   double dis_2d        // ground distance between the two
*********************************************************** */
double dist_2d_type::query_data(double freq, double alt_xmit, double alt_rec, double dis_2d) {

   interval_type *ptr;

   if (!intervalman.head)
      return 0.0;

   if (dis_2d < ((interval_type *)intervalman.head)->min)
      return ((interval_type *)intervalman.head)->data->query_data(freq, alt_xmit, alt_rec, dis_2d);

   for (ptr=(interval_type *)intervalman.head; ptr->next; ptr=(interval_type *)ptr->next)
      if (ptr->query_between(dis_2d))
         return ptr->data->query_data(freq, alt_xmit, alt_rec, dis_2d);
	 
   return ptr->data->query_data(freq, alt_xmit, alt_rec, dis_2d);
}


/* ***********************************************************
   This method just fills in the virtual function requirement for
   the dist_2d_type class.

   double *data,        // data range to search
   double *az_range,    // azimuth range to search
   double *el_range     // elevation range to search
*********************************************************** */
void dist_2d_type::query_minmax(double *data, double *az_range, double *el_range) {

}


/* ***********************************************************
   Meets parent requirement
*********************************************************** */
void dist_2d_type::extract_data(dbl_llist_manager *man, double *az, double *el) {

}


/* ***********************************************************
   This method returns the gain level of an irregular table

   double freq,         // frequency to search on
   double alt_xmit,     // transmitter altitude (MSL)
   double alt_rec,      // receiver altitude (MSL)
   double dis_2d        // ground distance between the two
*********************************************************** */
double gain_type::query_data(double freq, double alt_xmit, double alt_rec, double dis_2d) {

   return gain;
}


/* ***********************************************************
   This method returns the gain level of an irregular table

   double freq,   // frequency to search
   double az,     // azimuth to search
   double el      // elevation to search
*********************************************************** */
double gain_type::query_data(double freq, double az, double el) {

   return gain;
}


/* ***********************************************************
*********************************************************** */
void gain_type::extract_data(dbl_llist_manager *man, double *az, double *el) {

   azel_slice *ptr;
   
   man->append(ptr = new azel_slice, NULL);
   ptr->az = *az;
   ptr->el = *el;
   ptr->data = gain;
}


/* ***********************************************************
   This method dumps the gain level to stdout

   char *prefix  // tab string to print before this current block of data
*********************************************************** */
void gain_type::dump_data(char *prefix) {

   printf("%s\t %f\n", prefix, gain);
}


/* ***********************************************************
   This method extracts the gain level of an irregular table from a file

   FILE *infile,    // input file
   char *token,     // last processed token
   int *errorflag   // error flag
*********************************************************** */
void gain_type::extract_tdbdat(FILE *infile, char *token, int *errorflag) {

   gain = atof(token);
   *errorflag = get_sweg_token(infile, token);
}


/* ***********************************************************
   This method just fills in the virtual function requirement for
   the gain_type class.

   double *data,        // data range to search
   double *az_range,    // azimuth range to search
   double *el_range     // elevation range to search
*********************************************************** */
void gain_type::query_minmax(double *data, double *az_range, double *el_range) {

   if (data[0] > gain)
      data[0] = gain;
   if (data[1] < gain)
      data[1] = gain;	    
}


/* ***********************************************************
   This method returns the rcs level of an irregular table

   double freq,   // frequency to search
   double az,     // azimuth to search
   double el      // elevation to search
*********************************************************** */
double rcs_type::query_data(double freq, double az, double el) {

   return rcs;
}


/* ***********************************************************
*********************************************************** */
double rcs_type::query_data(double freq, double alt_xmit, double alt_rec, double dis_2d) {

   return rcs;
}


/* ***********************************************************
*********************************************************** */
void rcs_type::extract_data(dbl_llist_manager *man, double *az, double *el) {

   azel_slice *ptr;
   
   man->append(ptr = new azel_slice, NULL);
   ptr->az = *az;
   ptr->el = *el;
   ptr->data = rcs;
}


/* ***********************************************************
   This method dumps the rcs level to stdout

   char *prefix  // tab string to print before this current block of data
*********************************************************** */
void rcs_type::dump_data(char *prefix) {

   printf("%s\t %f\n", prefix, rcs);
}


/* ***********************************************************
   This method extracts the rcs level of an irregular table from a file

   FILE *infile,    // input file
   char *token,     // last processed token
   int *errorflag   // error flag
*********************************************************** */
void rcs_type::extract_tdbdat(FILE *infile, char *token, int *errorflag) {

   rcs = atof(token);
   *errorflag = get_sweg_token(infile, token);
}


/* ***********************************************************

   double *data,        // data range to search
   double *az_range,    // azimuth range to search
   double *el_range     // elevation range to search
*********************************************************** */
void rcs_type::query_minmax(double *data, double *az_range, double *el_range) {

   if (data[0] > rcs)
      data[0] = rcs;
   if (data[1] < rcs)
      data[1] = rcs;	    
}


/* ***********************************************************
*********************************************************** */
void pol_type::extract_tdbdat(FILE *infile, char *token, int *errorflag) {

   interval_type *ptr;
   interval_type *einterval = NULL;

   *errorflag = get_sweg_token(infile, token);
   
   while (*errorflag) {

      if (!strcmp(token, TOKEN_DIMENSION)) {
	 
         for (ptr=(interval_type *)intervalman.head; ptr; ptr=(interval_type *)ptr->next) {
            // level
            *errorflag = get_sweg_token(infile, token);
            // type
            *errorflag = get_sweg_token(infile, token);

            if (!strcmp(token, TOKEN_AZ))
               ptr->data = new azimuth_type;
            else if (!strcmp(token, TOKEN_EL))
               ptr->data = new elevation_type;
            else
               ptr->data = new frequency_type;

            ptr->data->extract_tdbdat(infile, token, errorflag);
         }
	 
         break;
      }

      intervalman.append(ptr = new interval_type, NULL);

      if (!strcmp(token, TOKEN_HORIZ_POL))
         ptr->type = POL_HORIZ;
      else if (!strcmp(token, TOKEN_VERT_POL))
         ptr->type = POL_VERT;
      else if (!strcmp(token, TOKEN_LEFT_CIR_POL))
         ptr->type = POL_LEFT_CIR;
      else if (!strcmp(token, TOKEN_RIGHT_CIR_POL))
         ptr->type = POL_RIGHT_CIR;
      else 
         ptr->type = POL_DEFAULT;

      *errorflag = get_sweg_token(infile, token);
   }

}


/* ***********************************************************
   Meets parent requirement

   double *data,        // data range to search
   double *az_range,    // azimuth range to search
   double *el_range     // elevation range to search
*********************************************************** */
void pol_type::query_minmax(double *data, double *az_range, double *el_range) {

}


/* ***********************************************************
   Meets parent requirement
*********************************************************** */
void pol_type::extract_data(dbl_llist_manager *man, double *az, double *el) {

}


/* ***********************************************************
   Meets parent requirement

   double freq,   // frequency to search
   double az,     // azimuth to search
   double el      // elevation to search
*********************************************************** */
double pol_type::query_data(double freq, double az, double el) {

   return 0;
}


/* ***********************************************************
*********************************************************** */
double pol_type::query_data(double freq, double alt_xmit, double alt_rec, double dis_2d) {

   return 0;
}


/* ***********************************************************
  This method extracts data from a susceptibility block in the TDB

   FILE *infile,    // input file
   char *token,     // last processed token
   int *errorflag   // error flag
*********************************************************** */
void susceptibility_type::extract_tdbdat(FILE *infile, char *token, int *errorflag) {

   *errorflag = get_sweg_token(infile, token);

   while (*errorflag) {

      if (!strcmp(token, TOKEN_SUSCEPTIBILITY)) {
         *errorflag = get_sweg_token(infile, token);
         return;
      }
      
      if (!strcmp(token, TOKEN_EMITTIVE_EM_SIG_TABLE)) {
      
         if (elist)
            delete elist;
	    
         *errorflag = get_sweg_token(infile, token);		// "DIMENSION"
         *errorflag = get_sweg_token(infile, token);		// #
         *errorflag = get_sweg_token(infile, token);		// type

         if (!strcmp(token, TOKEN_FREQ))
            elist = new frequency_type;
         else if (!strcmp(token, TOKEN_AZ))
            elist = new azimuth_type;
         else
            elist = new elevation_type;

         elist->extract_tdbdat(infile, token, errorflag);
         *errorflag = get_sweg_token(infile, token);		// emittive-em-sig-table
         *errorflag = get_sweg_token(infile, token);
         continue;
      }

      if (!strcmp(token, TOKEN_REFLECTIVE_EM_SIG_TABLE)) {
      
         if (rlist)
            delete rlist;
	    
         *errorflag = get_sweg_token(infile, token);		// "DIMENSION"
         *errorflag = get_sweg_token(infile, token);		// #
         *errorflag = get_sweg_token(infile, token);		// type

         if (!strcmp(token, TOKEN_FREQ))
            rlist = new frequency_type;
         else if (!strcmp(token, TOKEN_AZ))
            rlist = new azimuth_type;
         else
            rlist = new elevation_type;

         rlist->extract_tdbdat(infile, token, errorflag);
         *errorflag = get_sweg_token(infile, token);		// reflective-em-sig-table
         *errorflag = get_sweg_token(infile, token);
         continue;
      }

      *errorflag = get_sweg_token(infile, token);
   }

   *errorflag = get_sweg_token(infile, token);         
}


/* ***********************************************************
  This method looks for a specific irregular table

   char *x  // table to find
*********************************************************** */
susceptibility_type *susceptibility_type::find_table(char *x) {

   if (!name.stringcmp(x))
      return this;

   if (next)
      return ((susceptibility_type *)next)->find_table(x);
      
   return NULL;
}
